//
//  sample.m

#import "titlecase.h"
#import "Constants.h"
#import "VCTitleCase.h"
#import <AVFoundation/AVFoundation.h>


@implementation titlecase


- (id)init {
    self = [super init];
    if (self) {
        NSMenuItem *secondMenuItem = [[[NSMenuItem alloc] init] autorelease];
		[secondMenuItem setTitle:MKLocalized(@"Fix Spaces")];
		[secondMenuItem setTarget:self];
		[secondMenuItem setAction:@selector(justDashes:)];
        
		NSMenuItem *aMenuItem = [[[NSMenuItem alloc] init] autorelease];
		[aMenuItem setTitle:MKLocalized(@"Title Case")];
		[aMenuItem setTarget:self];
		[aMenuItem setAction:@selector(titleCase:)];
        
        NSMenuItem *durationItem = [[[NSMenuItem alloc] init] autorelease];
		[durationItem setTitle:MKLocalized(@"Duration to Hours")];
		[durationItem setTarget:self];
		[durationItem setAction:@selector(durationToHoursMinutes:)];
        
        NSMenuItem *alphaItem = [[[NSMenuItem alloc] init] autorelease];
		[alphaItem setTitle:MKLocalized(@"Languages and Subtitles Alphabetically")];
		[alphaItem setTarget:self];
		[alphaItem setAction:@selector(sortAlphabetically:)];
        
        
        MyPluginManager *pluginManager = [(MyControllerForPlugins *)[NSApp delegate] pluginManager];
		[pluginManager addMenuItem:secondMenuItem  menuTitle:MKLocalized(@"Fixes")];
        [pluginManager addMenuItem:aMenuItem menuTitle:MKLocalized(@"Fixes")];
        [pluginManager addMenuItem:durationItem menuTitle:MKLocalized(@"Fixes")];
        [pluginManager addMenuItem:alphaItem menuTitle:MKLocalized(@"Fixes")];
        
        // Manage linked files
        NSMenuItem *renameFile = [[[NSMenuItem alloc] init] autorelease];
		[renameFile setTitle:MKLocalized(@"Rename Linked File to Title")];
		[renameFile setTarget:self];
		[renameFile setAction:@selector(renameLinkedFile:)];
        // Example of how to add keyboard shortcuts
        //[renameFile setKeyEquivalent:@"L"];
        //[renameFile setKeyEquivalentModifierMask:NSCommandKeyMask];
        
        NSMenuItem *showLinkInFinder = [[[NSMenuItem alloc] init] autorelease];
		[showLinkInFinder setTitle:MKLocalized(@"Show in Finder")];
		[showLinkInFinder setTarget:self];
		[showLinkInFinder setAction:@selector(showLinkInFinder:)];
        
        NSMenuItem *createCover = [[[NSMenuItem alloc] init] autorelease];
		[createCover setTitle:MKLocalized(@"Create Cover from File")];
		[createCover setTarget:self];
		[createCover setAction:@selector(createCoverFromFile:)];
        
        [pluginManager addMenuItem:renameFile menuTitle:MKLocalized(@"Links")];
        [pluginManager addMenuItem:showLinkInFinder menuTitle:MKLocalized(@"Links")];
        [pluginManager addMenuItem:createCover menuTitle:MKLocalized(@"Links")];
        
	}
	return self;
}


#pragma mark - Linked File

- (IBAction)renameLinkedFile:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (NSManagedObject *next in selection) {
        NSString *title = [next valueForKey:@"title"];
        NSArray *links = [[next valueForKey:@"linksSet"] allObjects];
        links = [links sortedArrayUsingDescriptors:[(MyControllerForPlugins *)[NSApp delegate] positionSortDescriptors]];
        NSInteger i;
        for (i = 0; i < [links count]; i++) {
            NSManagedObject *aLink = [links objectAtIndex:i];
        NSString *path = [aLink valueForKey:@"url"];
        if (path) {
            path = [controller normalizeLinkedFiledPath:path];
            NSString *nameWithExtension = [path lastPathComponent];
            NSString *extension = [nameWithExtension pathExtension];
            NSString *name = [nameWithExtension stringByDeletingPathExtension];
            
            if (i) {
                title = [title stringByAppendingFormat:@" %ld", (long)i];
            }
            NSString *originalTitle = title;
      
            if ([title rangeOfString:@"/"].location != NSNotFound) {
                title = [title stringByReplacingOccurrencesOfString:@"/" withString:@"-"];
            }
            if ([title rangeOfString:@":"].location != NSNotFound) { //Although allowed, looks like a forward slash in Finder
                title = [title stringByReplacingOccurrencesOfString:@":" withString:@"-"];
            }
            

            if (title && [title isNotEqualTo:name]) {
                               
                NSString *newPath = [NSString stringWithFormat:@"%@/%@.%@", [path stringByDeletingLastPathComponent], title, extension];
                if ([[NSFileManager defaultManager] moveItemAtPath:path toPath:newPath error:nil]) {
                    
                    newPath = [@"file://" stringByAppendingString:newPath];
                    
                    if ([newPath hasPrefix:[NSString stringWithFormat:@"file://%@", [controller dataFolderPath]]]) {
                        NSMutableString *aNewURL = [NSMutableString stringWithString:newPath];
                        [aNewURL replaceOccurrencesOfString:[NSString stringWithFormat:@"file://%@", [(MyControllerForPlugins *)[NSApp delegate] dataFolderPath]] withString:@"[Data Folder]" options:NSLiteralSearch range:NSMakeRange(0,[aNewURL length])];
                        [aLink setValue:aNewURL forKey:@"url"];
                    }
                    else
                        [aLink setValue:newPath forKey:@"url"];
                    
                    [aLink setValue:originalTitle forKey:@"name"];
                }
            }
        }
            
        }
    }
}


- (IBAction)showLinkInFinder:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (NSManagedObject *next in selection) {
        NSManagedObject *aLink = [[next valueForKey:@"linksSet"] anyObject];
        NSString *path = [aLink valueForKey:@"url"];
        if (path) {
            path = [controller normalizeLinkedFiledPath:path];
            if ([[NSWorkspace sharedWorkspace] selectFile:path inFileViewerRootedAtPath:nil] == NO)
                [[NSWorkspace sharedWorkspace] openFile:[path stringByDeletingLastPathComponent]];
        }
    }
}



- (IBAction)createCoverFromFile:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (MOEntry *next in selection) {
        NSArray *links = [[next valueForKey:@"linksSet"] allObjects];
        links = [links sortedArrayUsingDescriptors:[controller positionSortDescriptors]];
        NSInteger i;
        for (i = 0; i < [links count]; i++) {
            NSManagedObject *aLink = [links objectAtIndex:i];
            NSString *path = [aLink valueForKey:@"url"];
            if (path) {
                path = [controller normalizeLinkedFiledPath:path];
                
                NSURL *url = [NSURL fileURLWithPath:path];
                
                NSDictionary *assetOptions = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:AVURLAssetPreferPreciseDurationAndTimingKey];
                AVAsset *localAsset = [AVURLAsset URLAssetWithURL:url options:assetOptions];
                if (localAsset) {
                    AVAssetImageGenerator *imageGenerator = [[AVAssetImageGenerator alloc] initWithAsset:localAsset];
                    
                    [localAsset loadValuesAsynchronouslyForKeys:[NSArray arrayWithObject:@"tracks"] completionHandler:^{
                        if ([localAsset statusOfValueForKey:@"tracks" error:NULL] != AVKeyValueStatusLoaded)
                            return;
                        
                        NSArray *visualTracks = [localAsset tracksWithMediaCharacteristic:AVMediaCharacteristicVisual];
                        NSArray *audibleTracks = [localAsset tracksWithMediaCharacteristic:AVMediaCharacteristicAudible];
                        if ([visualTracks count] > 0) {
                            // Grab the first frame from the asset and display it
                            [imageGenerator generateCGImagesAsynchronouslyForTimes:[NSArray arrayWithObject:[NSValue valueWithCMTime:CMTimeMake(10,1)]] completionHandler:^(CMTime requestedTime, CGImageRef image, CMTime actualTime, AVAssetImageGeneratorResult result, NSError *error) {
                                if (result == AVAssetImageGeneratorSucceeded) {
                                    NSImage *anImage = [[[NSImage alloc] initWithCGImage:image size:NSZeroSize] autorelease];
                                    [next setCoverImage:anImage];
                                    [controller performSelectorOnMainThread:@selector(refreshInformation) withObject:nil waitUntilDone:NO];
                                }
                                else
                                    NSLog(@"Error generating image.");
                            }];
                        }
                        else if ([audibleTracks count] > 0)
                        {
                            NSLog(@"Audio only asset.");
                        }
                        else
                        {
                            NSLog(@"Error Loafing asset tracks.");
                        }
                        
                    }];
                }
                else
                    NSLog(@"Error Loafing asset.");
            }
        }
    }
}


#pragma mark - Menu Command

- (IBAction)justDashes:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (NSManagedObject *next in selection) {
        NSString *originalTitle = [next valueForKey:@"title"];
        NSString *title = [originalTitle stringByReplacingOccurrencesOfString:@"_" withString:@" "];
        title = [title stringByReplacingOccurrencesOfString:@"-" withString:@" "];
        title = [title stringByReplacingOccurrencesOfString:@"  " withString:@" "];
        title = [title stringByReplacingOccurrencesOfString:@"  " withString:@" "];
        title = [title stringByRemovingLeadingAndTrailingSpaces];
        if ([title isNotEqualTo:originalTitle])
            [next setValue:title forKey:@"title"];
    }
    
}


- (IBAction)titleCase:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (NSManagedObject *next in selection) {
        NSString *originalTitle = [next valueForKey:@"title"];
        NSString *title = [originalTitle titlecaseString];
        title = [title stringByRemovingLeadingAndTrailingSpaces];
        title = [title stringByReplacingOccurrencesOfString:@"_" withString:@" "];
        if ([title isNotEqualTo:originalTitle])
            [next setValue:title forKey:@"title"];
    }
}


- (IBAction)durationToHoursMinutes:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (NSManagedObject *next in selection) {
        NSString *originalTitle = [next valueForKey:@"duration"];
        if ([originalTitle rangeOfString:@":"].location == NSNotFound) {
            NSInteger minutes = [originalTitle integerValue];
            NSInteger hours = minutes / 60;
            minutes = minutes % 60;
            if (hours)
                [next setValue:[NSString stringWithFormat:@"%ld:%ld", (long)hours, (long)minutes] forKey:@"duration"];
            else {
                [next setValue:[NSString stringWithFormat:@"%ld", (long)minutes] forKey:@"duration"];
            }
        }
    }
}


- (IBAction)sortAlphabetically:(id)sender {
    
    MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
    NSArray *selection = [controller selectedEntries];
    for (NSManagedObject *next in selection) {
        if ([next respondsToSelector:@selector(setLastSeen:)]) {
            NSArray *keys = @[@"languages", @"subtitles"];
            for (NSString *aKey in keys) {
                NSString *languages = [next valueForKey:aKey];
                NSArray *langArray = [languages componentsSeparatedByString:@", "];
                if ([langArray count] > 1) {
                    langArray = [langArray sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
                    NSString *sorted = [langArray componentsJoinedByString:@", "];
                    [next setValue:sorted forKey:aKey];
                }
            }
        }
    }
}


#pragma mark - Validation

- (BOOL)validateMenuItem:(NSMenuItem *)anItem {
    
    SEL action = [anItem action];
    NSString *actionString = NSStringFromSelector(action);
    if ([actionString isEqualToString:@"showLinkInFinder:"]
        || [actionString isEqualToString:@"renameLinkedFile:"]) {
        MyControllerForPlugins *controller = (MyControllerForPlugins *)[NSApp delegate];
        NSManagedObject *anEntry = [controller selectedEntry];
        if ([[anEntry valueForKey:@"linksSet"] count])
            return YES;
        return NO;
    }

    return YES;
}


@end
